using System.Text.Json;
using System.Threading.Channels;
using HermesSocketLibrary.db;
using HermesSocketLibrary.Requests;
using HermesSocketServer.Models;
using HermesSocketServer.Services;
using HermesSocketServer.Store;
using ILogger = Serilog.ILogger;

namespace HermesSocketServer.Requests
{
    public class UpdateTTSUser : IRequest
    {
        public string Name => "update_tts_user";

        private ChannelManager _channels;
        private Database _database;
        private readonly ServerConfiguration _configuration;
        private ILogger _logger;

        public UpdateTTSUser(ChannelManager channels, Database database, ServerConfiguration configuration, ILogger logger)
        {
            _database = database;
            _channels = channels;
            _configuration = configuration;
            _logger = logger;
        }


        public async Task<RequestResult> Grant(string sender, IDictionary<string, object>? data)
        {
            if (data == null)
            {
                _logger.Warning("Data received from request is null. Ignoring it.");
                return new RequestResult(false, null);
            }

            if (long.TryParse(data["chatter"].ToString(), out long chatterId))
                data["chatter"] = chatterId;
            if (data["voice"] is JsonElement v)
                data["voice"] = v.ToString();
            data["user"] = sender;

            var check = await _database.ExecuteScalar("SELECT state FROM \"TtsVoiceState\" WHERE \"userId\" = @user AND \"ttsVoiceId\" = @voice", data) ?? false;
            if ((check is not bool state || !state) && chatterId != _configuration.Tts.OwnerId)
                return new RequestResult(false, null);

            var channel = _channels.Get(sender);
            if (channel == null)
                return new RequestResult(false, null);

            channel.Chatters.Set(chatterId.ToString(), new ChatterVoice()
            {
                UserId = sender,
                ChatterId = chatterId.ToString(),
                VoiceId = data["voice"].ToString()!
            });
            _logger.Information($"Updated chatter's [chatter: {data["chatter"]}] selected tts voice [voice: {data["voice"]}] in channel [channel: {sender}]");
            return new RequestResult(true, null);
        }
    }
}